package digital.laboratory.platform.entrustment.service.impl;

import cn.hutool.core.collection.CollUtil;
import digital.laboratory.platform.common.core.exception.CheckedException;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.entrustment.entity.Entrustment;
import digital.laboratory.platform.entrustment.entity.EntrustmentIdentificationMaterial;
import digital.laboratory.platform.entrustment.enums.AnalysisOptionEnums;
import digital.laboratory.platform.entrustment.enums.EntrustIdentificationSituationType;
import digital.laboratory.platform.entrustment.service.EntrustmentIdentificationMaterialService;
import digital.laboratory.platform.entrustment.service.EntrustmentService;
import digital.laboratory.platform.entrustment.service.ExcelUploadService;
import digital.laboratory.platform.entrustment.utils.ExcelUtils;
import digital.laboratory.platform.sys.entity.DrugLite;
import digital.laboratory.platform.sys.enums.entrust.EntrustBiologyType;
import digital.laboratory.platform.sys.feign.RemoteCommDrugService;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author ChenJiangBao
 * @version 1.0
 * @description: excel上传服务层接口 实现类
 * @date 2025/4/16 11:55
 */
@Service
public class ExcelUploadServiceImpl implements ExcelUploadService {

    @Resource
    private EntrustmentService entrustmentService;

    @Resource
    private EntrustmentIdentificationMaterialService entrustmentIdentificationMaterialService;

    @Resource
    private RemoteCommDrugService remoteCommDrugService;

    /**
     * 上传两社人员名单excel文件
     * @param file
     * @param entrustId
     * @return
     */
    @Override
    public Boolean uploadHairInspectExcel(MultipartFile file, String entrustId) throws Exception {

        // 校验委托是否合法
        Entrustment entrustment = entrustmentService.getById(entrustId);
        if (entrustment == null) {
            throw new CheckedException(String.format("委托id为 %s 的委托信息不存在！"));
        }
        if (!entrustment.getOldIdentificationResult().equals(EntrustIdentificationSituationType.TWO_AGENCY.getDesc())) {
            throw new CheckedException("当前不支持两社人员之外的委托信息进行导入！");
        }
        // 默认取海洛因的毒品信息
        R<DrugLite> drugLiteR = remoteCommDrugService.getByName("海洛因");
        DrugLite heroin = drugLiteR.getData(); // 获取默认的海洛因数据
        R<List<DrugLite>> innerGetAllR = remoteCommDrugService.innerGetAll(); // 获取所有的毒品信息
        List<DrugLite> drugLiteList = innerGetAllR.getData();
        Map<String, DrugLite> drugLiteMap = drugLiteList.stream().collect(Collectors.toMap(DrugLite::getName, Function.identity()));
        // 防止检材列表
        List<EntrustmentIdentificationMaterial> entrustmentIdentificationMaterialList = new ArrayList<>();
        // 读取excel数据
        List<Map<String, String>> data = ExcelUtils.readExcel(file);
        for (Map<String, String> datum : data) {
            EntrustmentIdentificationMaterial entrustmentIdentificationMaterial = new EntrustmentIdentificationMaterial();
            entrustmentIdentificationMaterial.setEntrustmentId(entrustment.getId());
            entrustmentIdentificationMaterial.setType("1");
            entrustmentIdentificationMaterial.setTypeName("生物样本");
            entrustmentIdentificationMaterial.setName(datum.get("姓名") + "的毛发");
            entrustmentIdentificationMaterial.setBiologyGender(datum.get("性别"));
            entrustmentIdentificationMaterial.setBiologyType(EntrustBiologyType.HAIR.getDesc());
            entrustmentIdentificationMaterial.setMaterialAge(Integer.valueOf(datum.get("年龄")));
            entrustmentIdentificationMaterial.setRemark(datum.get("身份证号"));
            entrustmentIdentificationMaterial.setDrawPlace(datum.get("采样单位"));
            entrustmentIdentificationMaterial.setAnalysisOption(AnalysisOptionEnums.QUALITATIVE.getCode()); // 默认设置定性
            entrustmentIdentificationMaterial.setCandidateDrugs(CollUtil.newArrayList(drugLiteMap.containsKey(datum.get("曾吸毒种类")) ? drugLiteMap.get(datum.get("曾吸毒种类")) : heroin));
            entrustmentIdentificationMaterialList.add(entrustmentIdentificationMaterial);
        }

        return entrustmentIdentificationMaterialService.saveBatch(entrustmentIdentificationMaterialList);
    }
}
